     /**
      * \brief Vector Utilities
      *
      * This module contains several procedures which uses the vector
      * procedures for storing simple data.
      *
      * <br><br>
      *
      * An entry of the vector either contains only one data which can be
      * processed as is or multiple data which can be processed using the
      * datastructure templates in the copybook VUTIL_H.
      *
      * \author Mihael Schmidt
      * \date   24.07.2008
      */


      *-------------------------------------------------------------------------
      *
      * Copyright (c) 2007-2008 Mihael Schmidt
      * All rights reserved.
      *
      * This file is part of the VUTIL service program.
      *
      * VUTIL is free software: you can redistribute it and/or modify it under
      * the terms of the GNU Lesser General Public License as published by
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * VUTIL is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU Lesser General Public License for more details.
      *
      * You should have received a copy of the GNU Lesser General Public
      * License along with VUTIL.  If not, see http://www.gnu.org/licenses/.
      *
      *-------------------------------------------------------------------------


     HNOMAIN
     HBNDDIR('QC2LE')
     HOPTIMIZE(*NONE)
     HDEBUG(*YES)
     HOPTION(*SRCSTMT)
     HCOPYRIGHT('Copyright (c) 2007-2009 Mihael Schmidt. All rights reserved.')

      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
      /copy VECTOR_H
      /copy VUTIL_H
      /copy QUSLMBR_H
      /copy QUSLRCD_H
      /copy QUSLOBJ_H
      /copy QUSRJOBI_H
      /copy QUSLJOB_H
      /copy USRSPC_H
      /copy API_ERR_H
      /copy PARMEVAL_H


     /**
      * \brief List file member names
      *
      * All member names of a file are placed into a vector.
      * This vector is returned to the caller. On any error *null is
      * returned.
      *
      * \author Mihael Schmidt
      * \date   27.07.2008
      *
      * \param Library name
      * \param File name
      *
      * \return Pointer to the vector or *null if any error occurs
      */
     P vutil_listFileMembers...
     P                 B                   export
     D                 PI              *
     D   library                     10A   const
     D   filename                    10A   const
      *
     D vector          S               *
     D usPtr           S               *
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D usName          S             10A   inz('VUTIL1')
     D usLib           S             10A   inz('QTEMP')
     D dataPtr         S               *
     D usData          S             10A   based(dataPtr)
     D i               S             10I 0
      /free
       monitor;

         createUserSpace(usName + usLib : *blank : 4096 : X'00' : '*ALL' :
                         'List Util UserSpace' : '*YES' : apiErrDs );
         if (apiErrCode <> *blank);
           return *null;
         endif;

         // change user space attributes to autoextend
         userspace_attr.size = 1;       // 1=attribute will be changed
         userspace_attr.key = 3;        // 3=extensibility attribute
         userspace_attr.dataLength = 1;
         userspace_attr.data = '1';     // user space is extensible
         changeUserspaceAttr(usLib : usName + usLib : userspace_attr:apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         retrieveUserspacePtr(usName + usLib : usPtr : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         listFileMembers( usName + usLib : 'MBRL0100' : filename + library :
                          '*ALL' : '1' : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         vector = vector_create();

         // iterate through retrieved members
         for i = 0 to usHeader.nmbrEntries - 1;
           dataPtr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           vector_add(vector : %addr(usData) : %len(%trim(usData)));
         endfor;

         deleteUserspace(usName + usLib : apiErrDs);

         on-error *all;
           if (vector <> *null);
             vector_dispose(vector);
           endif;
           return *null;

       endmon;

       return vector;
      /end-free
     P                 E


     /**
      * \brief List record format names
      *
      * All record format names of a file are placed into a vector.
      * This vector is returned to the caller. On any error *null is
      * returned.
      *
      * \author Mihael Schmidt
      * \date   27.07.2008
      *
      * \param Library name
      * \param File name
      *
      * \return Pointer to the vector or *null if any error occured
      */
     P vutil_listRecordFormats...
     P                 B                   export
     D                 PI              *
     D   library                     10A   const
     D   filename                    10A   const
      *
     D vector          S               *
     D usPtr           S               *
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D usName          S             10A   inz('VUTIL2')
     D usLib           S             10A   inz('QTEMP')
     D dataPtr         S               *
     D usData          S             10A   based(dataPtr)
     D i               S             10I 0
      /free
       monitor;

         createUserSpace(usName + usLib : *blank : 4096 : X'00' : '*ALL' :
                         'List Util UserSpace' : '*YES' : apiErrDs );
         if (apiErrCode <> *blank);
           return *null;
         endif;

         // change user space attributes to autoextend
         userspace_attr.size = 1;       // 1=attribute will be changed
         userspace_attr.key = 3;        // 3=extensibility attribute
         userspace_attr.dataLength = 1;
         userspace_attr.data = '1';     // user space is extensible
         changeUserspaceAttr(usLib : usName + usLib : userspace_attr:apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         retrieveUserspacePtr(usName + usLib : usPtr : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         listRecordFormats( usName + usLib : 'RCDL0100' : filename + library :
                          '1' : apiErrDs);
         if (apiErrCode <> *blank);
           return *null;
         endif;

         vector = vector_create();

         // iterate through retrieved members
         for i = 0 to usHeader.nmbrEntries - 1;
           dataPtr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           vector_add(vector : %addr(usData) : %len(%trim(usData)));
         endfor;

         deleteUserspace(usName + usLib : apiErrDs);

         on-error *all;
           if (vector <> *null);
             vector_dispose(vector);
           endif;
           return *null;

       endmon;

       return vector;
      /end-free
     P                 E


     /**
      * \brief List objects
      *
      * All object names are placed into a vector.
      * This vector is returned to the caller. On any error *null is
      * returned.
      *
      * <br><br>
      *
      * The entries in the vector have the following format:
      * <ul>
      *   <li>10A Library name</li>
      *   <li>10A Object name</li>
      *   <li>10A Object type</li>
      * </ul>
      *
      * <br><br>
      * As this procedure utilizes the QUSLOBJ i5/OS API the parameters
      * here accept the same names and generic names for library, name and
      * object type.
      *
      * \param Library
      * \param Object name (default: *all)
      * \param Object type (default: *all)
      */
     P vutil_listObjects...
     P                 B                   export
     D                 PI              *
     D   pLibrary                    10A   const
     D   pObject                     10A   const options(*omit : *nopass)
     D   pType                       10A   const options(*nopass)
      *
     D lib             S             10A   inz('*ALL')
     D obj             S             10A   inz('*ALL')
     D type            S             10A   inz('*ALL')
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D usName          S             10A   inz('VUTIL3')
     D usLib           S             10A   inz('QTEMP')
     D usPtr           S               *
      *
     D ptr             S               *
     D i               S             10I 0
     D vector          S               *   inz(*null)
      /free
       lib = pLibrary;

       if (%parms() >= 2 and %addr(pObject) <> *null);
         obj = pObject;
       endif;

       if (%parms() = 3);
         type = pType;
       endif;

       // userspace erstellen
       createUserspace(usName + usLib :
                       'DATA' :
                       4096 :
                       X'00' :
                       '*ALL' :
                       %trimr(usName) + ' UserSpace' :
                       '*YES' :
                       apiErrDs );

       userspace_attr.size = 1;   // 1=attribute will be changed
       userspace_attr.key = 3;    // 3=extensibility attribute
       userspace_attr.dataLength = 1;
       userspace_attr.data = '1'; // user space is extensible
       changeUserspaceAttr(usLib : usName + usLib :
                           userspace_attr : apiErrDs);

       retrieveUserSpacePtr(usName + usLib : usPtr : apiErrDs);

       listObjects( usName + usLib :
                    'OBJL0100' :
                    obj + lib :
                    type :
                    apiErrDs );
       if (apiErrCode = *blank);
         vector = vector_create();

         for i = 0 to usHeader.nmbrEntries - 1;
           ptr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           vector_add(vector : ptr : 30);
         endfor;
       endif;

       // delete userspace
       deleteUserspace(usName + usLib : apiErrDs);

       return vector;
      /end-free
     P                 E


     /**
      * \brief List Job Library List
      *
      * All libraries of the library list of the specified job will be added
      * in the order of the library list returned in a vector.
      *
      * <br><br>
      *
      * The qualified job name consists of the following parts:
      * <ul>
      *   <li>CHAR (10) - Job Name</li>
      *   <li>CHAR (10) - User Name</li>
      *   <li>CHAR (6) - Job Number</li>
      * </ul>
      *
      * <br>
      *
      * If both parameters are passed then the library list of the job with
      * the qualified job name will be retrieved.
      *
      * \author Mihael Schmidt
      * \date   10.04.2008
      *
      * \param Qualified Job Name (optinal, default = current job)
      * \param Internal Job Number (optinal)
      * \param Library parts (see VUTIL_H Constants)
      *
      * \return Pointer to the vector or *null if any error occured
      */
     P vutil_listJobLibraryList...
     P                 B                   export
     D                 PI              *
     D   qualJobName                 26A   const options(*omit : *nopass)
     D   internalJobNumber...
     D                               16A   const options(*omit : *nopass)
     D   libParts                    10I 0 const options(*nopass)
      *
     D selLibParts     DS                  qualified
     D   system                        N
     D   product                       N
     D   current                       N
     D   user                          N
     D   selParts                     4A   overlay(selLibParts)
      *
     D jobi0700ds      DS                  qualified
     D   bytesReturned...
     D                               10I 0
     D   bytesAvailable...
     D                               10I 0
     D   jobname                     10A
     D   username                    10A
     D   jobnumber                    6A
     D   intJobNumber                16A
     D   jobstatus                   10A
     D   jobtype                      1A
     D   jobsubtype                   1A
     D   reserved                     2A
     D   numberSysLibs...
     D                               10I 0
     D   numberProdLibs...
     D                               10I 0
     D   numberCurrentLibs...
     D                               10I 0
     D   numberUserLibs...
     D                               10I 0
      *
     D offsetSystem    S             10I 0
     D offsetProduct   S             10I 0
     D offsetCurrent   S             10I 0
     D offsetUser      S             10I 0
      *
     D vector          S               *
     D receiver        S          65535A
     D libPtr          S               *
     D lib             S             11A   based(libPtr)
     D i               S             10I 0
      /free
       apiErrDs = *blank;

       if (%parms() = 3);
         selLibParts.selParts = evalParm(libParts);
       else;
         selLibParts.selParts = evalParm(VUTIL_ALL_LIBS);
       endif;

       monitor;
         if (%parms() >= 1 and %addr(qualJobName) <> *null);
           retrieveJobInformation(
               receiver :
               %len(receiver) :
               'JOBI0700' :
               qualJobName :
               '' :
               apiErrDs);
         elseif (%parms() >= 2 and %addr(internalJobNumber) <> *null);
           retrieveJobInformation(
               receiver :
               %len(receiver) :
               'JOBI0700' :
               '*INT' :
               internalJobNumber :
               apiErrDs);
         else;
           retrieveJobInformation(
               receiver :
               %len(receiver) :
               'JOBI0700' :
               '*' :
               '' :
               apiErrDs);
         endif;

         on-error *all;
           return *null;
       endmon;

       if (apiErrCode = *blank);
         jobi0700ds = receiver;

         vector = vector_create();

         // calculate offsets
         offsetSystem = 80;
         offsetProduct = offsetSystem + jobi0700ds.numberSysLibs * 11;
         offsetCurrent = offsetProduct + jobi0700ds.numberProdLibs * 11;
         offsetUser = offsetCurrent + jobi0700ds.numberCurrentLibs * 11;

         // add system libraries
         if (selLibParts.system);
           libPtr = %addr(receiver) + offsetSystem;
           for i = 0 to jobi0700ds.numberSysLibs - 1;
             vector_add(vector : libPtr : %len(%trimr(lib)));
             libPtr = libPtr + 11;
           endfor;
         endif;

         // add product libraries
         if (selLibParts.product);
           libPtr = %addr(receiver) + offsetProduct;
           for i = 0 to jobi0700ds.numberProdLibs - 1;
             vector_add(vector : libPtr : %len(%trimr(lib)));
             libPtr = libPtr + 11;
           endfor;
         endif;

         // add current library
         if (selLibParts.current and jobi0700ds.numberCurrentLibs > 0);
           libPtr = %addr(receiver) + offsetCurrent;
           vector_add(vector : libPtr : %len(%trimr(lib)));
         endif;

         // add user libraries
         if (selLibParts.user);
           libPtr = %addr(receiver) + offsetUser;
           for i = 0 to jobi0700ds.numberUserLibs - 1;
             vector_add(vector : libPtr : %len(%trimr(lib)));
             libPtr = libPtr + 11;
           endfor;
         endif;
       endif;

       return vector;
      /end-free
     P                 E


     /**
      * \brief List Active Jobs in Subsystem
      *
      * All active jobs of the specified subsystem will be added to the vector.
      * The entries of the vector are qualified job names which consist of the
      * following parts:
      * <ul>
      *   <li>CHAR (10) - Job Name</li>
      *   <li>CHAR (10) - User Name</li>
      *   <li>CHAR (6) - Job Number</li>
      * </ul>
      *
      * The datastructure template <em>vutil_tmpl_job</em> can be used for
      * the entries.
      *
      * <br>
      *
      * Subsystem monitor jobs are exluded.
      *
      * \author Mihael Schmidt
      * \date   29.04.2008
      *
      * \param Subsystemdescription library
      * \param Subsystemdescription name
      *
      * \return Pointer to the vector or *null if any error occured
      *
      * \info The user calling this procedure must have *JOBCTL
      *       authorities else this procedure returns *null.
      */
     P vutil_listActiveSubsystemJobs...
     P                 B                   export
     D                 PI              *
     D  sbsLib                       10A   const
     D  sbsName                      10A   const
      *
     D cSbsName        C                   1906
      *
     D vector          S               *   inz(*null)
     D usLib           S             10A   inz('QTEMP')
     D usName          S             10A   inz('VUTIL4')
     D usPtr           S               *
     D usHeader        DS                  likeds(userspace_gen) based(usPtr)
     D i               S             10I 0
      *
     D jobPtr          S               *
     D jobDs           DS                  qualified based(jobPtr)
     D   name                        10A
     D   user                        10A
     D   number                       6A
     D   internalId                  16A
     D   status                      10A
     D   type                         1A
     D   subtype                      1A
     D   reserved1                    2A
     D   infoStatus                   1A
     D   reserved2                    3A
     D   fieldsReturned...
     D                               10I 0
     D   keyArray                     1A
      *
     D keyArrPtr       S               *
     D jobKeyArray     DS                  qualified based(keyArrPtr)
     D   bytesReturned...
     D                               10I 0
     D   keyField                    10I 0
     D   dataType                     1A
     D   reserved                     3A
     D   dataLength                  10I 0
     D   data                        20A
      /free
       apiErrSiz = 0;
       // userspace erstellen
       createUserspace(usName + usLib :
                       'DATA' :
                       4096 :
                       X'00' :
                       '*ALL' :
                       %trimr(usName) + ' UserSpace' :
                       '*YES' :
                       apiErrDs );

       apiErrSiz = 0;
       userspace_attr.size = 1;   // 1=attribute will be changed
       userspace_attr.key = 3;    // 3=extensibility attribute
       userspace_attr.dataLength = 1;
       userspace_attr.data = '1'; // user space is extensible
       changeUserspaceAttr(usLib : usName + usLib :
                           userspace_attr : apiErrDs);

       apiErrSiz = 0;
       retrieveUserSpacePtr(usName + usLib : usPtr : apiErrDs);

       apiErrSiz = 200;
       listJobs(usName + usLib :
                'JOBL0200' :
                '*ALL      *ALL      *ALL':
                '*ACTIVE' :
                apiErrDs :
                '*' :
                1 :
                cSbsName);
       if (apiErrCode = *blank);
         vector = vector_create();

         for i = 0 to usHeader.nmbrEntries - 1;
           jobPtr = usPtr + usHeader.offsetList + (i * usHeader.entrySize);
           keyArrPtr = %addr(jobDs.keyArray);

           if (jobKeyArray.keyField <> cSbsName or
               jobKeyArray.dataType <> 'C' or
               jobKeyArray.dataLength <> 20);
             // TODO send message to job log
             vector_dispose(vector);
             vector = *null;
             leave;
           endif;

           // only list jobs in requested subsystem
           // and ignore subsystem monitor jobs
           if (jobKeyArray.data = sbsName + sbsLib and
               jobDs.type <> 'M');
             vector_add(vector : %addr(jobDs) : 26);
           endif;
         endfor;
       endif;

       // delete userspace
       apiErrSiz = 250;
       deleteUserspace(usName + usLib : apiErrDs);

       return vector;
      /end-free
     P                 E
